/***************************************************************************
                          confparser.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Willy De la Court
    email                : willy.delacourt@pandora.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2 as     *
 *   published by the Free Software Foundation                             *
 *                                                                         *
 ***************************************************************************/

#include <qfile.h>
#include <qtextstream.h>
#include <klocale.h>
#include <kmessagebox.h>
#include "confparser.h"
#include "mlistviewitem.h"
#include "server.h"
#include "items.h"

extern int conf_yyparse(void *);
extern int conf_yydebug;
extern int conf_yypos;
extern int conf_lineno;
extern const char *conf_lexinput;

ConfParser::ConfParser(QString fileName, MListViewItem *serverItem)
{
	thisFileName = fileName;
	thisServerItem = serverItem;
}

bool ConfParser::Parse()
{
	if ( KIO::NetAccess::download(thisFileName, tmpFileName) )
	{
		//qDebug(tmpFileName.latin1());
		QString result;
		hostDataItem *thisHost;
		rangesDataItem *thisRanges;
		subnetDataItem *thisSubNet;
		optionsItem *listGlobalOptions, *thisOptions;
		addSubnetDataItem *listSubNets;
		addHostDataItem *thisHosts;
		subnetItem *elSubNet;
		hostItem *elHost;
		QFile configFile( tmpFileName );

		if (configFile.open( IO_ReadOnly ))
		{
			QTextStream ts(&configFile);
			ts.setEncoding(QTextStream::Latin1); // Locale??
			configData = ts.read();
			configFile.close();
			conf_lexinput = configData.ascii();
			conf_yydebug = 0;
			conf_yypos = 0;
			conf_lineno = 1;
			conf_yyparse( this );
			//result = generate();
			//printf("%s", result.ascii());
			listGlobalOptions = new optionsItem(thisServerItem, i18n("Global Options"), &globalOptions, 0);
			listSubNets = new addSubnetDataItem(thisServerItem, i18n("Subnets"), &subnets, 0);
			// TODO Groups management
			//listGroups = new MListViewItem(thisServerItem, i18n("Groups"), 0);

			for ( elSubNet = subnets.first(); elSubNet; elSubNet = subnets.next() )
			{
				thisSubNet = new subnetDataItem(listSubNets->thisListViewItem, i18n("Subnet ") + elSubNet->itemName, &subnets, elSubNet, 0);
				thisRanges = new rangesDataItem(thisSubNet->thisListViewItem, i18n("Ranges"), &(elSubNet->ranges), 0);
				thisOptions = new optionsItem(thisSubNet->thisListViewItem, i18n("Options"), &(elSubNet->subnetOptions), 0);
				thisHosts = new addHostDataItem(thisSubNet->thisListViewItem, i18n("Hosts"), &(elSubNet->hosts), 0);
				for ( elHost = elSubNet->hosts.first(); elHost; elHost = elSubNet->hosts.next() )
					thisHost = new hostDataItem(thisHosts->thisListViewItem, elHost->itemHostName, &(elSubNet->hosts), elHost, 0);
			}
			return(true);
		}
		return(false);
	}
	else
	{
		KMessageBox::sorry(0, i18n("Unable to read: ") + thisFileName);
		return(false);
	}
}

void ConfParser::addSyntaxError(int lineno)
{
	int *x = new int(lineno);
	syntaxErrors.append( x );
}

void ConfParser::addGlobal(dataItem *data)
{
	globalOptions.append(data);
}

void ConfParser::addSubnet(subnetItem *subnet)
{
	subnets.append(subnet);
}

QString ConfParser::generate()
{
	QString result;
	dataItem *element;

	for ( element = globalOptions.first(); element; element = globalOptions.next() )
		result += element->generate("");
	for ( element = subnets.first(); element; element = subnets.next() )
		result += element->generate("");
	return result;
}

void ConfParser::save()
{
	QString result = generate();
	QFile configFile( tmpFileName );

	if (configFile.open( IO_WriteOnly ))
	{
		QTextStream ts(&configFile);
		ts.setEncoding(QTextStream::Latin1); // Locale??
		ts << result;
		configFile.close();
		KIO::NetAccess::upload(tmpFileName, thisFileName);
		setChanged( false );
	}
	//printf("%s", result.ascii());
}

bool ConfParser::isChanged()
{
	bool state;
	dataItem *element;

	state = false;
	for ( element = globalOptions.first(); element; element = globalOptions.next() )
		state |= element->isChanged();
	for ( element = subnets.first(); element; element = subnets.next() )
		state |= element->isChanged();
	return state;
}

void ConfParser::setChanged(bool state)
{
	dataItem *element;

	for ( element = globalOptions.first(); element; element = globalOptions.next() )
		element->setChanged( state );
	for ( element = subnets.first(); element; element = subnets.next() )
		element->setChanged( state );
}

ConfParser::~ConfParser()
{
	globalOptions.clear();
	subnets.clear();
}
