/***************************************************************************
                          dataclasses.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Willy De la Court
    email                : willy.delacourt@pandora.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2 as     *
 *   published by the Free Software Foundation                             *
 *                                                                         *
 ***************************************************************************/

#include <qregexp.h>
#include "dataclasses.h"

dataItem::dataItem(const char *name, const char *data, const char *type, bool quotes)
{
	itemName = name;
	itemData = data;
	itemType = type;
	hasQuotes = quotes;
	changed = false;
}

void dataItem::setComment(const char *comment)
{
	QString tostrip( comment );
	tostrip.replace( QRegExp( "^#" ), "" );
	itemComments.append( tostrip.stripWhiteSpace() );
}

void dataItem::setData(const char *data)
{
	itemData = data;
}

QString dataItem::generateComment(QString ident, bool makeComment)
{
	QString result;

	QStringList::iterator it;
    	for ( it = itemComments.begin(); it != itemComments.end(); ++it )
	{
		if ( makeComment )
			result += ident + "# " + (*it).ascii();
		else
			result += ident + (*it).ascii();
		result += "\n";
	}
	if ( itemComments.isEmpty() )
		result += "\n";
	return result;
}

QString dataItem::generate(QString ident)
{
	QString result;

	if( hasQuotes )
		result = ident + itemName + " " + "\"" + itemData + "\";";
	else
		result = ident + itemName + " " + itemData + ";";
	result += generateComment(ident);
	return result;
}

bool dataItem::isChanged()
{
	return changed;
}

void dataItem::setChanged( bool state )
{
	changed = state;
}

rangeItem::rangeItem(const char *ip1, const char *ip2, const char *comment, bool bootp)
	: dataItem("range", "")
{
	rangeIP1 = ip1;
	rangeIP2 = ip2;
	itemComments += QStringList::split( QRegExp( "\\n" ), comment );
	rangeBootp = bootp;
}

rangeItem::rangeItem(const char *ip1, const char *ip2, bool bootp)
	: dataItem("range", "")
{
	rangeIP1 = ip1;
	rangeIP2 = ip2;
	rangeBootp = bootp;
}

QString rangeItem::generate(QString ident)
{
	QString result;

	if ( rangeBootp )
		result = ident + "range dynamic-bootp " + rangeIP1 + " " + rangeIP2 + ";";
	else
		result = ident + "range " + rangeIP1 + " " + rangeIP2 + ";";
	result += generateComment(ident);
	return result;
}

optionItem::optionItem(const char *name, const char *data, bool quotes)
	: dataItem(name, data, "simple", quotes)
{
}

QString optionItem::generate(QString ident)
{
	QString result;

	if( hasQuotes )
		result = ident + "option " + itemName + " " + "\"" + itemData + "\";";
	else
		result = ident + "option " + itemName + " " + itemData + ";";
	result += generateComment(ident);
	return result;
}

hostItem::hostItem(const char *name)
	: dataItem("host", name)
{
	itemHostName = name;
}

void hostItem::addOption(dataItem *option)
{
	hostOptions.append(option);
}

QString hostItem::generate(QString ident)
{
	QString result;
	dataItem *element;

	result = ident + "host " + itemHostName + " {";
	result += generateComment(ident = "\t");
	for ( element = hostOptions.first(); element; element = hostOptions.next() )
		result += element->generate(ident + "\t");
	result += ident + "}\n";
	return result;
}

bool hostItem::isChanged()
{
	dataItem *element;
	bool state;

	state = changed;
	for ( element = hostOptions.first(); element; element = hostOptions.next() )
		state |= element->isChanged();
	return state;
}

void hostItem::setChanged(bool state)
{
	dataItem *element;

	changed = state;
	for ( element = hostOptions.first(); element; element = hostOptions.next() )
		element->setChanged( state );
}

subnetItem::subnetItem(const char *subnet, const char *netmask)
	: dataItem(subnet, netmask)
{
	itemSubnet = subnet;
	itemNetmask = netmask;
}

QString subnetItem::generate(QString ident)
{
	QString result;
	dataItem *element;

	result = ident + "subnet " + itemSubnet + " netmask " + itemNetmask + " {";
	result += generateComment(ident + "\t");
	for ( element = subnetOptions.first(); element; element = subnetOptions.next() )
		result += element->generate(ident + "\t");
	for ( element = ranges.first(); element; element = ranges.next() )
		result += element->generate(ident + "\t");
	for ( element = hosts.first(); element; element = hosts.next() )
		result += element->generate(ident + "\t");
	result += ident + "}\n";
	return result;
}

void subnetItem::addOption(dataItem *option)
{
	subnetOptions.append(option);
}

void subnetItem::addRange(rangeItem *range)
{
	ranges.append(range);
}

void subnetItem::addHost(hostItem *host)
{
	hosts.append(host);
}

bool subnetItem::isChanged()
{
	bool state;
	dataItem *element;

	state = changed;
	for ( element = subnetOptions.first(); element; element = subnetOptions.next() )
		state |= element->isChanged();
	for ( element = ranges.first(); element; element = ranges.next() )
		state |= element->isChanged();
	for ( element = hosts.first(); element; element = hosts.next() )
		state |= element->isChanged();
	return state;
}

void subnetItem::setChanged( bool state )
{
	dataItem *element;

	changed = state;
	for ( element = subnetOptions.first(); element; element = subnetOptions.next() )
		element->setChanged( state );
	for ( element = ranges.first(); element; element = ranges.next() )
		element->setChanged( state );
	for ( element = hosts.first(); element; element = hosts.next() )
		element->setChanged( state );
}

leaseItem::leaseItem(const char *ip)
	: dataItem("lease", ip)
{
	itemAbandoned = false;
	itemIp = ip;
}

QString leaseItem::generate(QString ident)
{
	QString result;
	dataItem *element;

	return result;
}


